#
# @(#)$Id: PdfPlugin.pm,v 1.4 2003/04/10 10:56:49 ssrpkl Exp $ (C) 2002 by Peter Klausner
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, published at 
# http://www.gnu.org/copyleft/gpl.html
#
# =========================

package TWiki::Plugins::PdfPlugin;

# =========================
use vars qw(
        $web $topic $user $installWeb $VERSION $debug
        $exampleCfgVar
    );

$VERSION = '1.000';

# =========================
sub initPlugin
{
    ( $topic, $web, $user, $installWeb ) = @_;

    # check for Plugins.pm versions
    if( $TWiki::Plugins::VERSION < 1 ) {
        &TWiki::Func::writeWarning( "Version mismatch between $Id: PdfPlugin.pm,v 1.4 2003/04/10 10:56:49 ssrpkl Exp $ and Plugins.pm" );
        return 0;
    }

    # Get plugin preferences, the variable defined by:          * Set EXAMPLE = ...
    #$exampleCfgVar = &TWiki::Prefs::getPreferencesValue( "EMPTYPLUGIN_EXAMPLE" ) || "default";

    # Get plugin debug flag
    $debug = &TWiki::Func::getPreferencesFlag( "PDFPLUGIN_DEBUG" );

    # Plugin correctly initialized
    &TWiki::Func::writeDebug( "- TWiki::Plugins::PdfPlugin::initPlugin( $web.$topic ) is OK" ) if $debug;
    return 1;
}


# =========================
###sub startRenderingHandler
#{
#    $_[0] =~	s/%TOC%//g;	# that interferes with HTMLDOC
#}
# patched handleToc, because above does not work!

# =========================
sub endRenderingHandler
{
    return	unless $ENV{"QUERY_STRING"} =~ "\bskin=pdf";

    # turn TWiki's relative links into absolute ones...
    $_[0] =~	s(href="/$web/)
		 (href="$TWiki::defaultUrlHost/$web/)go;
    $_[0] =~	s(href="$TWiki::scriptUrlPath/)
		 (href="$TWiki::defaultUrlHost$TWiki::scriptUrlPath/)go;
}
	
# =========================
sub convert2pdf
{
 	my ( $tmpl, $topic, $web, $rev, $revdate, $revuser ) = @_;

	my $pdfcall = &TWiki::Store::readTemplate( "pdfcall", $skin );
        
	# Put title page in tmp, if needed:
	my $titlefile = "/tmp/twiki2pdf.title.$$.htm";
	if ( $pdfcall =~ / --title/ )
	{
	  my $pdftitle = &TWiki::Store::readTemplate( "pdftitle", $skin );
          
	  # htmldoc discards block up to first header; use this as abstract.
	  my $abstract = "no abstract given in first paragraph";
	  if ( $tmpl =~ s/(<body[^>]*>)(.*?)(<h1)/$1$3/si ) {
	      $abstract = $2;
	  }
	  else {	# try to find first block with minimum markup <a <b> <em>
	      my $headline =
	          TWiki::Plugins::SpacedWikiWordPlugin::spacedWikiWord( $topic )
			or $headline = $topic;
	      $tmpl =~ s|(<body[^>]*>)(.*?)<([^abe/])|$1<h1>$headline</h1>$2<$3|si and
	      $abstract = substr($2, 0, 512);
	  }
	  $pdftitle =~ s/%ABSTRACT%/$abstract/go;
	  $pdftitle =~ s/%REVINFO%/r1.$rev - $revdate GMT $revuser/go;
	  $pdftitle =~ s/%REVDATE%/$revdate/go;
	  $pdftitle =~ s/%REVUSER%/$revuser/go;
	  $pdftitle =~ s/%AUTHORLIST%/&getAuthorList($web, $topic, $revuser)/geo;
	  $pdftitle =~ s/%REV%/$rev/go;
	  $pdftitle = &TWiki::handleCommonTags( $pdftitle, $topic );
	  $pdftitle = &TWiki::getRenderedVersion( $pdftitle, "" );
	  open TITLE, ">$titlefile"	or die "open $titlefile: $!";
	  print TITLE "$pdftitle"	or die "print $titlefile: $!";
	  close TITLE			or die "close $titlefile: $!";
	  $pdfcall =~ s/%PDFTITLE%/$titlefile/go;
	}
	# else body template takes care of title block at the beginning

	# Put actual text into tmp.
	# fix links
	$tmpl =~ s(href="$TWiki::scriptUrlPath/)
		  (href="$TWiki::defaultUrlHost$TWiki::scriptUrlPath/)go;
	my $bodyfile = "/tmp/twiki2pdf.body.$$.htm";
	open BODY, ">$bodyfile"		or die "open $bodyfile: $!";
	print BODY "$tmpl"		or die "print $bodyfile: $!";
	close BODY			or die "close $bodyfile: $!";
	$pdfcall =~ s/%PDFBODY%/$bodyfile/go;

	# Go for it.
	$| = 1;	# Make sure header gets written before system output!
        print "Content-type: application/pdf\n\n";	
	$| = 0;
	$ENV{"PATH"} = "";
	$pdfcall =~ m|^(/[^`]*)|;
	system("$1") or TWiki::Func::writeDebug( "pdf - error: $!" );
###	unlink $bodyfile, $titlefile;
	exit 0;
}
	
# =========================
sub getAuthorList	# <web> <topic> <latest revision user>
{			# tbd: $_[1] gets spaced out - use global $topic :*(
	my $authors = "";
	my %cnt;
	$cnt{$_[2]} = 3;	# give latest author a count bonus
	open RLOG,
	"$TWiki::rcsDir/rlog $TWiki::rcsArgs $TWiki::dataDir/$web/$topic.txt|"
		or return "$_[2]";
	while (<RLOG>)	{
		/^date:.* author: (\w+)/   and ++$cnt{"$TWiki::mainWebname.$1"};
	}
	close RLOG;
	# tbd: sort by number of submission instead of alphabetically:
	for (sort (keys %cnt)) { $authors .= $authors? ", $_" : $_; }
	return $authors;
}

1;
